/*
 *  nautilus-search-tool.c
 * 
 *  Copyright (C) 2005-2009 Andrea Zagli <azagli@libero.it>
 *
 *  This file is part of nautilus-search-tool.
 *  
 *  nautilus-search-tool is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  nautilus-search-tool is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with nautilus-search-tool; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

#ifdef HAVE_CONFIG_H
	#include <config.h> /* for GETTEXT_PACKAGE */
#endif

#include "nautilus-search-tool.h"

#include <libnautilus-extension/nautilus-menu-provider.h>

#include <glib/gi18n-lib.h>

static void nautilus_search_tool_instance_init (NautilusSearchTool *cvs);
static void nautilus_search_tool_class_init    (NautilusSearchToolClass *class);

static GType search_tool_type = 0;
static gchar *gnome_search_tool = NULL;

static void
search_tool_callback (NautilusMenuItem *item,
                      NautilusFileInfo *file_info)
{
	gchar *uri;
	gchar *argv[3];
	gchar *working_directory;

	uri = nautilus_file_info_get_uri (file_info);

	if (uri)
		{
			if (strcmp (nautilus_file_info_get_uri_scheme (file_info), "x-nautilus-desktop") == 0)
				{
					working_directory = g_strdup (g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP));
				}
			else
				{
					working_directory = g_filename_from_uri (uri, NULL, NULL);
					if (!nautilus_file_info_is_directory (file_info)) {
						gchar *tmp;

						tmp = g_path_get_dirname (working_directory);
						g_free (working_directory);
						working_directory = tmp;
					}
				}
		}
	else
		{
			working_directory = g_strdup (g_get_home_dir ());
		}

	g_free (uri);

	argv[0] = gnome_search_tool;

	argv[1] = g_strdup_printf ("--path=%s", working_directory);
	argv[2] = NULL;

	g_spawn_async (working_directory,
	               argv,
	               NULL,
	               0,
	               NULL,
	               NULL,
	               NULL,
	               NULL);

	g_free (working_directory);
	g_free (argv[1]);
}

static NautilusMenuItem *
search_tool_menu_item_new ()
{
	return nautilus_menu_item_new ("NautilusSearchTool::search_tool",
	                               _("Search for Files..."),
	                               _("Find files, folders, and documents on selected position"),
	                               "gnome-searchtool");
}

static GList *
nautilus_search_tool_get_background_items (NautilusMenuProvider *provider,
                                           GtkWidget            *window,
                                           NautilusFileInfo     *file_info)
{
	NautilusMenuItem *item;
	if (!gnome_search_tool)
		return NULL;

	item = search_tool_menu_item_new ();
	g_signal_connect (item, "activate",
	                  G_CALLBACK (search_tool_callback),
	                  file_info);

	return g_list_append (NULL, item);
}

GList *
nautilus_search_tool_get_file_items (NautilusMenuProvider *provider,
                                     GtkWidget            *window,
                                     GList                *files)
{
	NautilusMenuItem *item;

	if (!gnome_search_tool)
		return NULL;

	if (g_list_length (files) != 1)
		return NULL;

	item = search_tool_menu_item_new ();
	g_signal_connect (item, "activate",
	                  G_CALLBACK (search_tool_callback),
	                  files->data);

	return g_list_append (NULL, item);
}

static void
nautilus_search_tool_menu_provider_iface_init (NautilusMenuProviderIface *iface)
{
	iface->get_background_items = nautilus_search_tool_get_background_items;
	iface->get_file_items = nautilus_search_tool_get_file_items;
}

static void 
nautilus_search_tool_instance_init (NautilusSearchTool *cvs)
{
}

static void
nautilus_search_tool_class_init (NautilusSearchToolClass *class)
{
}

GType
nautilus_search_tool_get_type (void) 
{
	return search_tool_type;
}

void
nautilus_search_tool_register_type (GTypeModule *module)
{
	static const GTypeInfo info = {
		sizeof (NautilusSearchToolClass),
		(GBaseInitFunc) NULL,
		(GBaseFinalizeFunc) NULL,
		(GClassInitFunc) nautilus_search_tool_class_init,
		NULL, 
		NULL,
		sizeof (NautilusSearchTool),
		0,
		(GInstanceInitFunc) nautilus_search_tool_instance_init,
	};

	static const GInterfaceInfo menu_provider_iface_info = {
		(GInterfaceInitFunc) nautilus_search_tool_menu_provider_iface_init,
		NULL,
		NULL
	};

	search_tool_type = g_type_module_register_type (module,
	                                                G_TYPE_OBJECT,
	                                                "NautilusSearchTool",
	                                                &info, 0);

	g_type_module_add_interface (module,
	                             search_tool_type,
	                             NAUTILUS_TYPE_MENU_PROVIDER,
	                             &menu_provider_iface_info);

	gnome_search_tool = g_find_program_in_path ("gnome-search-tool");
}

void
nautilus_search_tool_free (void)
{
	g_free (gnome_search_tool);
	gnome_search_tool = NULL;
}
